@@ -36,6 +36,7 @@ namespace Twitterizer
 {
     using System;
     using System.Diagnostics;
+    using System.Runtime.Serialization;
     using Newtonsoft.Json;
     using Twitterizer.Core;
 
@@ -44,35 +45,19 @@ namespace Twitterizer
     /// </summary>
     [JsonObject(MemberSerialization = MemberSerialization.OptIn)]
     [DebuggerDisplay("TwitterList = {FullName}")]
+#if !SILVERLIGHT
     [Serializable]
+#endif
+    [DataContract]
     public class TwitterList : TwitterObject
     {
-        #region Constructors
-        /// <summary>
-        /// Initializes a new instance of the <see cref="TwitterList"/> class.
-        /// </summary>
-        /// <param name="tokens">OAuth access tokens.</param>
-        public TwitterList(OAuthTokens tokens) 
-            : base()
-        {
-            this.Tokens = tokens;
-        }
-
-        /// <summary>
-        /// Initializes a new instance of the <see cref="TwitterList"/> class.
-        /// </summary>
-        public TwitterList()
-            : base()
-        {
-        }
-        #endregion
-
         #region API properties
         /// <summary>
         /// Gets or sets the id.
         /// </summary>
         /// <value>The list id.</value>
         [JsonProperty(PropertyName = "id")]
+        [DataMember]
         public long Id { get; set; }
 
         /// <summary>
@@ -80,6 +65,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The list name.</value>
         [JsonProperty(PropertyName = "name")]
+        [DataMember]
         public string Name { get; set; }
 
         /// <summary>
@@ -87,6 +73,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The full name.</value>
         [JsonProperty(PropertyName = "full_name")]
+        [DataMember]
         public string FullName { get; set; }
 
         /// <summary>
@@ -94,6 +81,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The list slug.</value>
         [JsonProperty(PropertyName = "slug")]
+        [DataMember]
         public string Slug { get; set; }
 
         /// <summary>
@@ -101,6 +89,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The description.</value>
         [JsonProperty(PropertyName = "description")]
+        [DataMember]
         public string Description { get; set; }
 
         /// <summary>
@@ -108,6 +97,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The number of subscribers.</value>
         [JsonProperty(PropertyName = "subscriber_count")]
+        [DataMember]
         public int NumberOfSubscribers { get; set; }
 
         /// <summary>
@@ -115,6 +105,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The number of members.</value>
         [JsonProperty(PropertyName = "member_count")]
+        [DataMember]
         public int NumberOfMembers { get; set; }
 
         /// <summary>
@@ -122,6 +113,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The absolute path.</value>
         [JsonProperty(PropertyName = "uri")]
+        [DataMember]
         public string AbsolutePath { get; set; }
 
         /// <summary>
@@ -129,6 +121,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The list mode.</value>
         [JsonProperty(PropertyName = "mode")]
+        [DataMember]
         public string Mode { get; set; }
 
         /// <summary>
@@ -136,6 +129,7 @@ namespace Twitterizer
         /// </summary>
         /// <value>The owning user.</value>
         [JsonProperty(PropertyName = "user")]
+        [DataMember]
         public TwitterUser User { get; set; }
         #endregion
 
@@ -144,6 +138,7 @@ namespace Twitterizer
         /// Gets a value indicating whether this instance is public.
         /// </summary>
         /// <value><c>true</c> if this instance is public; otherwise, <c>false</c>.</value>
+        [DataMember]
         public bool IsPublic
         {
             get
@@ -163,18 +158,48 @@ namespace Twitterizer
         /// <param name="description">The description.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterList"/> instance.</returns>
-        public static TwitterList New(OAuthTokens tokens, string username, string name, bool isPublic, string description, OptionalProperties options)
+        [Obsolete("The username parameter is no longer required.")]
+        public static TwitterResponse<TwitterList> New(OAuthTokens tokens, string username, string name, bool isPublic, string description, OptionalProperties options)
         {
-            Commands.CreateListCommand command = new Twitterizer.Commands.CreateListCommand(tokens, name, username, options)
+            return New(tokens, name, isPublic, description, options);
+        }
+
+        /// <summary>
+        /// Creates a new list for the authenticated user. Accounts are limited to 20 lists.
+        /// </summary>
+        /// <param name="tokens">The oauth tokens.</param>
+        /// <param name="name">The list name.</param>
+        /// <param name="isPublic">if set to <c>true</c> creates a public list.</param>
+        /// <param name="description">The description.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        public static TwitterResponse<TwitterList> New(OAuthTokens tokens, string name, bool isPublic, string description, OptionalProperties options)
+        {
+            Commands.CreateListCommand command = new Twitterizer.Commands.CreateListCommand(tokens, name, options)
             {
                 IsPublic = isPublic,
                 Description = description
             };
 
-            return Core.CommandPerformer<TwitterList>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
+        /// Creates a new list for the authenticated user. Accounts are limited to 20 lists.
+        /// </summary>
+        /// <param name="tokens">The oauth tokens.</param>
+        /// <param name="username">The username.</param>
+        /// <param name="name">The list name.</param>
+        /// <param name="isPublic">if set to <c>true</c> creates a public list.</param>
+        /// <param name="description">The description.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        /// <remarks></remarks>
+        public static TwitterResponse<TwitterList> New(OAuthTokens tokens, string username, string name, bool isPublic, string description)
+        {
+            return New(tokens, name, isPublic, description, null);
+        }
+
+         /// <summary>
         /// Updates the specified list.
         /// </summary>
         /// <param name="tokens">The oauth tokens.</param>
@@ -182,61 +207,90 @@ namespace Twitterizer
         /// <param name="listId">The list id.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterList"/> instance.</returns>
-        public static TwitterList Update(OAuthTokens tokens, string username, long listId, UpdateListOptions options)
+        [Obsolete("The username parameter is no longer required.")]
+        public static TwitterResponse<TwitterList> Update(OAuthTokens tokens, string username, string listId, UpdateListOptions options)
         {
-            Commands.UpdateListCommand command = new Twitterizer.Commands.UpdateListCommand(tokens, username, listId, options);
+            return Update(tokens, listId, options);
+        }
 
-            return Core.CommandPerformer<TwitterList>.PerformAction(command);
+        /// <summary>
+        /// Updates the specified list.
+        /// </summary>
+        /// <param name="tokens">The oauth tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        /// <remarks></remarks>
+        public static TwitterResponse<TwitterList> Update(OAuthTokens tokens, string listId, UpdateListOptions options)
+        {
+            Commands.UpdateListCommand command = new Twitterizer.Commands.UpdateListCommand(tokens, listId, options);
+
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
         /// List the lists of the specified user. Private lists will be included if the authenticated users is the same as the user who's lists are being returned.
         /// </summary>
         /// <param name="tokens">The tokens.</param>
-        /// <param name="username">The username.</param>
         /// <param name="options">The options.</param>
         /// <returns>
         /// A <see cref="TwitterListCollection"/> instance.
         /// </returns>
-        public static TwitterListCollection GetLists(OAuthTokens tokens, string username, OptionalProperties options)
+        public static TwitterResponse<TwitterListCollection> GetLists(OAuthTokens tokens, GetListsOptions options = null)
         {
-            Commands.GetListsCommand command = new Twitterizer.Commands.GetListsCommand(tokens, username, options);
-            TwitterListCollection results = Core.CommandPerformer<TwitterListCollection>.PerformAction(command);
+            Commands.GetListsCommand command = new Twitterizer.Commands.GetListsCommand(tokens, options);
 
-            if (results != null)
-                results.Command = command;
-
-            return results;
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
-        /// List the lists of the specified user. Private lists will be included if the authenticated users is the same as the user who's lists are being returned.
+        /// Returns the specified list. Private lists will only be shown if the authenticated user owns the specified list.
         /// </summary>
         /// <param name="tokens">The tokens.</param>
-        /// <param name="username">The username.</param>
-        /// <returns>
-        /// A <see cref="TwitterListCollection"/> instance.
-        /// </returns>
-        public static TwitterListCollection GetLists(OAuthTokens tokens, string username)
+        /// <param name="slug">The slug.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        public static TwitterResponse<TwitterList> Show(OAuthTokens tokens, string slug)
         {
-            return GetLists(tokens, username, null);
+            return Show(tokens, slug, null);
         }
 
         /// <summary>
-        /// Show the specified list. Private lists will only be shown if the authenticated user owns the specified list.
+        /// Returns the specified list. Private lists will only be shown if the authenticated user owns the specified list.
         /// </summary>
         /// <param name="tokens">The tokens.</param>
-        /// <param name="username">The username.</param>
-        /// <param name="listIdOrSlug">The list id or slug.</param>
+        /// <param name="slug">The slug.</param>
         /// <param name="options">The options.</param>
-        /// <returns>
-        /// A <see cref="TwitterListCollection"/> instance.
-        /// </returns>
-        public static TwitterList GetList(OAuthTokens tokens, string username, string listIdOrSlug, OptionalProperties options)
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        public static TwitterResponse<TwitterList> Show(OAuthTokens tokens, string slug, OptionalProperties options)
         {
-            Commands.GetListCommand command = new Twitterizer.Commands.GetListCommand(tokens, username, listIdOrSlug, options);
+            Commands.GetListCommand command = new Twitterizer.Commands.GetListCommand(tokens, slug, -1, options);
 
-            return Core.CommandPerformer<TwitterList>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Returns the specified list. Private lists will only be shown if the authenticated user owns the specified list.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        public static TwitterResponse<TwitterList> Show(OAuthTokens tokens, decimal listId)
+        {
+            return Show(tokens, listId, null);
+        }
+
+        /// <summary>
+        /// Returns the specified list. Private lists will only be shown if the authenticated user owns the specified list.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>A <see cref="TwitterList"/> instance.</returns>
+        public static TwitterResponse<TwitterList> Show(OAuthTokens tokens, decimal listId, OptionalProperties options)
+        {
+            Commands.GetListCommand command = new Twitterizer.Commands.GetListCommand(tokens, string.Empty, listId, options);
+
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -247,11 +301,11 @@ namespace Twitterizer
         /// <param name="listIdOrSlug">The list id or slug.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterList"/> instance.</returns>
-        public static TwitterList Delete(OAuthTokens tokens, string username, string listIdOrSlug, OptionalProperties options)
+        public static TwitterResponse<TwitterList> Delete(OAuthTokens tokens, string username, string listIdOrSlug, OptionalProperties options)
         {
             Commands.DeleteListCommand command = new Twitterizer.Commands.DeleteListCommand(tokens, username, listIdOrSlug, options);
 
-            return Core.CommandPerformer<TwitterList>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -264,11 +318,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatusCollection"/> instance.
         /// </returns>
-        public static TwitterStatusCollection GetStatuses(OAuthTokens tokens, string username, string listIdOrSlug, ListStatusesOptions options)
+        public static TwitterResponse<TwitterStatusCollection> GetStatuses(OAuthTokens tokens, string username, string listIdOrSlug, ListStatusesOptions options)
         {
             Commands.ListStatusesCommand command = new Twitterizer.Commands.ListStatusesCommand(tokens, username, listIdOrSlug, options);
 
-            return Core.CommandPerformer<TwitterStatusCollection>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -280,14 +334,10 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterListCollection"/> instance.
         /// </returns>
-        public static TwitterListCollection GetMemberships(OAuthTokens tokens, string username, OptionalProperties options)
+        public static TwitterResponse<TwitterListCollection> GetMemberships(OAuthTokens tokens, string username, ListMembershipsOptions options)
         {
             Commands.ListMembershipsCommand command = new Twitterizer.Commands.ListMembershipsCommand(tokens, username, options);
-            TwitterListCollection result = Core.CommandPerformer<TwitterListCollection>.PerformAction(command);
-
-            result.Command = command;
-
-            return result;
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -298,7 +348,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterListCollection"/> instance.
         /// </returns>
-        public static TwitterListCollection GetMemberships(OAuthTokens tokens, string username)
+        public static TwitterResponse<TwitterListCollection> GetMemberships(OAuthTokens tokens, string username)
         {
             return GetMemberships(tokens, username, null);
         }
@@ -312,11 +362,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterListCollection"/> instance.
         /// </returns>
-        public static TwitterListCollection GetSubscriptions(OAuthTokens tokens, string userName, OptionalProperties options)
+        public static TwitterResponse<TwitterListCollection> GetSubscriptions(OAuthTokens tokens, string userName, GetListSubscriptionsOptions options)
         {
             Commands.GetListSubscriptionsCommand command = new Twitterizer.Commands.GetListSubscriptionsCommand(tokens, userName, options);
 
-            return Core.CommandPerformer<TwitterListCollection>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -327,7 +377,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterListCollection"/> instance.
         /// </returns>
-        public static TwitterListCollection GetSubscriptions(OAuthTokens tokens, string userName)
+        public static TwitterResponse<TwitterListCollection> GetSubscriptions(OAuthTokens tokens, string userName)
         {
             return GetSubscriptions(tokens, userName, null);
         }
@@ -343,14 +393,11 @@ namespace Twitterizer
         /// <returns>
         /// A collection of users as <see cref="TwitterUserCollection"/>.
         /// </returns>
-        public static TwitterUserCollection GetMembers(OAuthTokens tokens, string username, string listIdOrSlug, GetListMembersOptions options)
+        public static TwitterResponse<TwitterUserCollection> GetMembers(OAuthTokens tokens, string username, string listIdOrSlug, GetListMembersOptions options)
         {
             Commands.GetListMembersCommand command = new Twitterizer.Commands.GetListMembersCommand(tokens, username, listIdOrSlug, options);
 
-            TwitterUserCollection result = CommandPerformer<TwitterUserCollection>.PerformAction(command);
-            result.CursorPagedCommand = command;
-
-            return result;
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -358,11 +405,10 @@ namespace Twitterizer
         /// </summary>
         /// <param name="tokens">The tokens.</param>
         /// <param name="username">The username.</param>
-        /// <param name="listId">The list id.</param>
-        /// <returns>
-        /// A collection of users as <see cref="TwitterUserCollection"/>.
-        /// </returns>
-        public static TwitterUserCollection GetMembers(OAuthTokens tokens, string username, string listIdOrSlug)
+        /// <param name="listIdOrSlug">The list id or slug.</param>
+        /// <returns>A collection of users as <see cref="TwitterUserCollection"/>.</returns>
+        /// <remarks></remarks>
+        public static TwitterResponse<TwitterUserCollection> GetMembers(OAuthTokens tokens, string username, string listIdOrSlug)
         {
             return GetMembers(tokens, username, listIdOrSlug, null);
         }
@@ -378,11 +424,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterList"/> representing the list the user was added to, or <c>null</c>.
         /// </returns>
-        public static TwitterList AddMember(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userIdToAdd, OptionalProperties options)
+        public static TwitterResponse<TwitterList> AddMember(OAuthTokens tokens, string ownerUsername, string listId, decimal userIdToAdd, OptionalProperties options)
         {
             Commands.AddListMemberCommand command = new Twitterizer.Commands.AddListMemberCommand(tokens, ownerUsername, listId, userIdToAdd, options);
 
-            return CommandPerformer<TwitterList>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -395,7 +441,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterList"/> representing the list the user was added to, or <c>null</c>.
         /// </returns>
-        public static TwitterList AddMember(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userIdToAdd)
+        public static TwitterResponse<TwitterList> AddMember(OAuthTokens tokens, string ownerUsername, string listId, decimal userIdToAdd)
         {
             return AddMember(tokens, ownerUsername, listId, userIdToAdd, null);
         }
@@ -411,11 +457,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterList"/> representing the list the user was added to, or <c>null</c>.
         /// </returns>
-        public static TwitterList RemoveMember(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userIdToAdd, OptionalProperties options)
+        public static TwitterResponse<TwitterList> RemoveMember(OAuthTokens tokens, string ownerUsername, string listId, decimal userIdToAdd, OptionalProperties options)
         {
             Commands.RemoveListMemberCommand command = new Twitterizer.Commands.RemoveListMemberCommand(tokens, ownerUsername, listId, userIdToAdd, options);
 
-            return CommandPerformer<TwitterList>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -428,7 +474,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterList"/> representing the list the user was added to, or <c>null</c>.
         /// </returns>
-        public static TwitterList RemoveMember(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userIdToAdd)
+        public static TwitterResponse<TwitterList> RemoveMember(OAuthTokens tokens, string ownerUsername, string listId, decimal userIdToAdd)
         {
             return RemoveMember(tokens, ownerUsername, listId, userIdToAdd, null);
         }
@@ -444,7 +490,7 @@ namespace Twitterizer
         /// <returns>
         /// The user's details, if they are a member of the list, otherwise <c>null</c>.
         /// </returns>
-        public static TwitterUser CheckMembership(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userId, OptionalProperties options)
+        public static TwitterResponse<TwitterUser> CheckMembership(OAuthTokens tokens, string ownerUsername, string listId, decimal userId, OptionalProperties options)
         {
             Commands.CheckListMembershipCommand command = new Twitterizer.Commands.CheckListMembershipCommand(
                 tokens,
@@ -453,7 +499,7 @@ namespace Twitterizer
                 userId,
                 options);
 
-            return CommandPerformer<TwitterUser>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -466,9 +512,49 @@ namespace Twitterizer
         /// <returns>
         /// The user's details, if they are a member of the list, otherwise <c>null</c>.
         /// </returns>
-        public static TwitterUser CheckMembership(OAuthTokens tokens, string ownerUsername, decimal listId, decimal userId)
+        public static TwitterResponse<TwitterUser> CheckMembership(OAuthTokens tokens, string ownerUsername, string listId, decimal userId)
         {
             return CheckMembership(tokens, ownerUsername, listId, userId, null);
         }
+
+        /// <summary>
+        /// Subscribes the specified tokens.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterList> Subscribe(OAuthTokens tokens, decimal listId)
+        {
+            return Subscribe(tokens, listId, null);
+        }
+
+        /// <summary>
+        /// Subscribes the specified tokens.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <param name="optionalProperties">The optional properties.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterList> Subscribe(OAuthTokens tokens, decimal listId, OptionalProperties optionalProperties)
+        {
+            Commands.CreateListMembershipCommand command = new Commands.CreateListMembershipCommand(tokens, listId, optionalProperties);
+
+            return CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Unsubscribes the authenticated user from the specified list.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="listId">The list id.</param>
+        /// <param name="optionalProperties">The optional properties.</param>
+        /// <returns></returns>
+        /// <remarks></remarks>
+        public static TwitterResponse<TwitterList> UnSubscribe(OAuthTokens tokens, decimal listId, OptionalProperties optionalProperties)
+        {
+            Commands.DestroyListSubscriber command = new Commands.DestroyListSubscriber(tokens, listId, optionalProperties);
+
+            return CommandPerformer.PerformAction(command);
+        }
     }
 }
