@@ -68,6 +68,12 @@ namespace Smuxi.Engine
         
         public override string NetworkID {
             get {
+                if (!String.IsNullOrEmpty(_JabberClient.NetworkHost)) {
+                    return _JabberClient.NetworkHost;
+                }
+                if (!String.IsNullOrEmpty(_JabberClient.Server)) {
+                    return _JabberClient.Server;
+                }
                 return "XMPP";
             }
         }
@@ -128,6 +134,8 @@ namespace Smuxi.Engine
             Host = server.Hostname;
             Port = server.Port;
 
+            ApplyConfig(Session.UserConfig, server);
+
             // TODO: use config for single network chat or once per network manager
             _NetworkChat = Session.CreateChat<ProtocolChatModel>(
                 NetworkID, "Jabber " + Host, this
@@ -135,8 +143,14 @@ namespace Smuxi.Engine
             Session.AddChat(_NetworkChat);
             Session.SyncChat(_NetworkChat);
 
-            ApplyConfig(Session.UserConfig, server);
-
+            if (!String.IsNullOrEmpty(_JabberClient.ProxyHost)) {
+                var builder = CreateMessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(_("Using proxy: {0}:{1}"),
+                                   _JabberClient.ProxyHost,
+                                   _JabberClient.ProxyPort);
+                Session.AddMessageToChat(Chat, builder.ToMessage());
+            }
             _JabberClient.Connect();
         }
         
@@ -411,6 +425,11 @@ namespace Smuxi.Engine
         
         public void CommandJoin(CommandModel cd)
         {
+            if (cd.DataArray.Length < 2) {
+                NotEnoughParameters(cd);
+                return;
+            }
+
             string jid = cd.DataArray[1];
             ChatModel chat = GetChat(jid, ChatType.Group);
             if (chat == null) {
