@@ -81,6 +81,9 @@ namespace Smuxi.Engine
             // setting required default values
             prefix = "Frontend/";
             LoadEntry(prefix+"UseLowBandwidthMode", false);
+            LoadEntry(prefix+"ShowQuickJoin", true);
+            LoadEntry(prefix+"ShowMenuBar", true);
+            LoadEntry(prefix+"ShowStatusBar", true);
 
             prefix = "Frontend/Engines/";
             Get<string[]>(prefix+"Engines", new string[] {});
