@@ -18,13 +18,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
-#if INDICATE_SHARP
+#if INDICATE_SHARP || MESSAGING_MENU_SHARP
 using System;
 using System.IO;
 using System.Text.RegularExpressions;
 using System.Collections.Generic;
 using System.Runtime.InteropServices;
+#if INDICATE_SHARP
 using Indicate;
+#elif MESSAGING_MENU_SHARP
+using MessagingMenu;
+#endif
 #if IPC_DBUS
     #if DBUS_SHARP
 using DBus;
@@ -44,13 +48,18 @@ namespace Smuxi.Frontend.Gnome
         private static readonly log4net.ILog Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
         const string BusName = "com.canonical.indicator.session";
-        private static DateTime UnixEpoch = new DateTime(1970, 1, 1, 0, 0, 0);
         private static string PersonChatIconBase64 { get; set; }
         private static string GroupChatIconBase64  { get; set; }
+#if INDICATE_SHARP
         Server Server { get; set; }
+        Dictionary<ChatView, Indicator> Indicators { get; set; }
+#elif MESSAGING_MENU_SHARP
+        private static DateTime UnixEpoch = new DateTime(1970, 1, 1, 0, 0, 0);
+        App App { get; set; }
+        Dictionary<ChatView, string> Sources { get; set; }
+#endif
         MainWindow MainWindow { get; set; }
         ChatViewManager ChatViewManager { get; set; }
-        Dictionary<ChatView, Indicator> Indicators { get; set; }
         Dictionary<ChatView, MessageTextViewMessageHighlightedEventHandler> HighlightEventHandlers { get; set; }
         bool IsInitialized { get; set; }
         bool IsEnabled { get; set; }
@@ -80,7 +89,11 @@ namespace Smuxi.Frontend.Gnome
 
             MainWindow = mainWindow;
             ChatViewManager = chatViewManager;
+#if INDICATE_SHARP
             Indicators = new Dictionary<ChatView, Indicator>();
+#elif MESSAGING_MENU_SHARP
+            Sources = new Dictionary<ChatView, string>();
+#endif
             HighlightEventHandlers = new Dictionary
                 <ChatView,
                  MessageTextViewMessageHighlightedEventHandler>();
@@ -104,8 +117,12 @@ namespace Smuxi.Frontend.Gnome
             ChatViewManager.ChatAdded   -= OnChatViewManagerChatAdded;
             ChatViewManager.ChatRemoved -= OnChatViewManagerChatRemoved;
 
+#if INDICATE_SHARP
             Server.Hide();
-            Server.Dispose();
+#elif MESSAGING_MENU_SHARP
+            App.Unregister();
+            App.Dispose();
+#endif
         }
 
         public void ApplyConfig(UserConfig userConfig)
@@ -145,7 +162,11 @@ namespace Smuxi.Frontend.Gnome
                 path = Path.Combine(path, "smuxi-frontend-gnome");
                 File.WriteAllText(path, DesktopFile + "\n");
 
+#if INDICATE_SHARP
                 Server.Show();
+#elif MESSAGING_MENU_SHARP
+                App.Register();
+#endif
                 IsEnabled = true;
                 // hide the main window instead of closing it
                 MainWindow.NotificationAreaIconMode = NotificationAreaIconMode.Closed;
@@ -155,11 +176,103 @@ namespace Smuxi.Frontend.Gnome
                 // https://wiki.ubuntu.com/MessagingMenu/#Registration
                 File.WriteAllText(blacklistPath, DesktopFile + "\n");
 
+#if INDICATE_SHARP
                 Server.Hide();
+#elif MESSAGING_MENU_SHARP
+                App.Unregister();
+#endif
                 IsEnabled = false;
             }
         }
 
+        void OnMainWindowFocusInEvent(object sender, Gtk.FocusInEventArgs e)
+        {
+            if (MainWindow.Notebook.IsBrowseModeEnabled) {
+                return;
+            }
+
+            var currentChatView = MainWindow.Notebook.CurrentChatView;
+            if (currentChatView == null) {
+                return;
+            }
+#if INDICATE_SHARP
+            DisposeIndicator(currentChatView);
+#elif MESSAGING_MENU_SHARP
+            DisposeSource(currentChatView);
+#endif
+        }
+
+        void OnMainWindowNotebookSwitchPage(object sender, Gtk.SwitchPageArgs e)
+        {
+            if (MainWindow.Notebook.IsBrowseModeEnabled) {
+                return;
+            }
+
+            var currentChatView = MainWindow.Notebook.CurrentChatView;
+            if (currentChatView == null) {
+                return;
+            }
+#if INDICATE_SHARP
+            DisposeIndicator(currentChatView);
+#elif MESSAGING_MENU_SHARP
+            DisposeSource(currentChatView);
+#endif
+        }
+
+        void OnChatViewManagerChatAdded(object sender, ChatViewManagerChatAddedEventArgs e)
+        {
+            // we are only interested in highlights on person and group chats
+            if (!(e.ChatView is PersonChatView) &&
+                !(e.ChatView is GroupChatView)) {
+                return;
+            }
+
+            MessageTextViewMessageHighlightedEventHandler handler =
+            delegate(object o, MessageTextViewMessageHighlightedEventArgs args) {
+                    OnChatViewMessageHighlighted(o, args, e.ChatView);
+            };
+            e.ChatView.OutputMessageTextView.MessageHighlighted += handler;
+
+            // keep a reference to the handler so we can cleanup it up later
+            // in OnChatViewManagerChatRemoved()
+            HighlightEventHandlers.Add(e.ChatView, handler);
+        }
+
+        void OnChatViewManagerChatRemoved(object sender, ChatViewManagerChatRemovedEventArgs e)
+        {
+            MessageTextViewMessageHighlightedEventHandler handler;
+            if (!HighlightEventHandlers.TryGetValue(e.ChatView, out handler)) {
+                return;
+            }
+
+            e.ChatView.OutputMessageTextView.MessageHighlighted -= handler;
+
+            // close possibly active indicator
+#if INDICATE_SHARP
+            DisposeIndicator(e.ChatView);
+#elif MESSAGING_MENU_SHARP
+            DisposeSource(e.ChatView);
+#endif
+        }
+
+        void OnChatViewMessageHighlighted(object sender,
+                                          MessageTextViewMessageHighlightedEventArgs e,
+                                          ChatView chatView)
+        {
+            if (!IsEnabled ||
+                e.Message.TimeStamp <= chatView.SyncedLastSeenHighlight ||
+                MainWindow.HasToplevelFocus) {
+                return;
+            }
+
+#if INDICATE_SHARP
+            ShowIndicator(chatView, e.Message);
+#elif MESSAGING_MENU_SHARP
+            ShowSource(chatView, e.Message);
+#endif
+        }
+
+#if INDICATE_SHARP
         void Init()
         {
             Trace.Call();
@@ -221,91 +334,18 @@ namespace Smuxi.Frontend.Gnome
             IsInitialized = true;
         }
 
-        void OnMainWindowFocusInEvent(object sender, Gtk.FocusInEventArgs e)
+        void ShowIndicator(ChatView chatView, MessageModel msg)
         {
-            if (MainWindow.Notebook.IsBrowseModeEnabled) {
-                return;
-            }
-
-            var currentChatView = MainWindow.Notebook.CurrentChatView;
-            if (currentChatView == null) {
-                return;
-            }
-            DisposeIndicator(currentChatView);
-        }
-
-        void OnMainWindowNotebookSwitchPage(object sender, Gtk.SwitchPageArgs e)
-        {
-            if (MainWindow.Notebook.IsBrowseModeEnabled) {
-                return;
-            }
-
-            var currentChatView = MainWindow.Notebook.CurrentChatView;
-            if (currentChatView == null) {
-                return;
-            }
-            DisposeIndicator(currentChatView);
-        }
-
-        void OnServerServerDisplay(object sender, ServerDisplayArgs e)
-        {
-            Trace.Call(sender, e);
-
-            MainWindow.PresentWithTime(e.Timestamp);
-        }
-
-        void OnChatViewManagerChatAdded(object sender, ChatViewManagerChatAddedEventArgs e)
-        {
-            // we are only interested in highlights on person and group chats
-            if (!(e.ChatView is PersonChatView) &&
-                !(e.ChatView is GroupChatView)) {
-                return;
-            }
-
-            MessageTextViewMessageHighlightedEventHandler handler =
-            delegate(object o, MessageTextViewMessageHighlightedEventArgs args) {
-                    OnChatViewMessageHighlighted(o, args, e.ChatView);
-            };
-            e.ChatView.OutputMessageTextView.MessageHighlighted += handler;
-
-            // keep a reference to the handler so we can cleanup it up later
-            // in OnChatViewManagerChatRemoved()
-            HighlightEventHandlers.Add(e.ChatView, handler);
-        }
-
-        void OnChatViewManagerChatRemoved(object sender, ChatViewManagerChatRemovedEventArgs e)
-        {
-            MessageTextViewMessageHighlightedEventHandler handler;
-            if (!HighlightEventHandlers.TryGetValue(e.ChatView, out handler)) {
-                return;
-            }
-
-            e.ChatView.OutputMessageTextView.MessageHighlighted -= handler;
-
-            // close possibly active indicator
-            DisposeIndicator(e.ChatView);
-        }
-
-        void OnChatViewMessageHighlighted(object sender,
-                                          MessageTextViewMessageHighlightedEventArgs e,
-                                          ChatView chatView)
-        {
-            if (!IsEnabled ||
-                e.Message.TimeStamp <= chatView.SyncedLastSeenHighlight ||
-                MainWindow.HasToplevelFocus) {
-                return;
-            }
-
             Indicator indicator;
             if (Indicators.TryGetValue(chatView, out indicator)) {
                 // update time of existing indicator
                 indicator.SetProperty(
                     "time",
-                    e.Message.TimeStamp.ToLocalTime().ToString("s")
+                    msg.TimeStamp.ToLocalTime().ToString("s")
                 );
                 return;
             }
-            
+
             indicator = new Indicator();
             indicator.SetProperty("subtype", "im");
             if (chatView is PersonChatView) {
@@ -314,7 +354,7 @@ namespace Smuxi.Frontend.Gnome
             }
             if (chatView is GroupChatView) {
                 indicator.SetProperty("icon", GroupChatIconBase64);
-                var nick = GetNick(e.Message);
+                var nick = GetNick(msg);
                 if (nick == null) {
                     indicator.SetProperty("sender", chatView.Name);
                 } else {
@@ -328,7 +368,7 @@ namespace Smuxi.Frontend.Gnome
             }
             indicator.SetProperty(
                 "time",
-                e.Message.TimeStamp.ToLocalTime().ToString("s")
+                msg.TimeStamp.ToLocalTime().ToString("s")
             );
             indicator.SetPropertyBool("draw-attention", true);
             indicator.UserDisplay += delegate {
@@ -378,6 +418,134 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        void OnServerServerDisplay(object sender, ServerDisplayArgs e)
+        {
+            Trace.Call(sender, e);
+
+            MainWindow.PresentWithTime(e.Timestamp);
+        }
+
+#elif MESSAGING_MENU_SHARP
+        void Init()
+        {
+            Trace.Call();
+
+#if IPC_DBUS
+            if (!Bus.Session.NameHasOwner(BusName)) {
+    #if LOG4NET
+                Logger.Info("Init(): no DBus provider for messaging menu found, " +
+                            "disabling...");
+                return;
+            }
+    #endif
+#endif
+
+            App = new App("smuxi-frontend-gnome.desktop");
+            App.ActivateSource += OnAppActivateSource;
+
+            MainWindow.FocusInEvent += OnMainWindowFocusInEvent;
+            MainWindow.Notebook.SwitchPage += OnMainWindowNotebookSwitchPage;
+
+            ChatViewManager.ChatAdded   += OnChatViewManagerChatAdded;
+            ChatViewManager.ChatRemoved += OnChatViewManagerChatRemoved;
+
+            IsInitialized = true;
+        }
+
+        void ShowSource(ChatView chatView, MessageModel msg)
+        {
+            Trace.Call(chatView, msg);
+
+            string sourceId;
+            var time = (Int64) ((msg.TimeStamp - UnixEpoch).TotalMilliseconds * 1000L);
+            if (Sources.TryGetValue(chatView, out sourceId)) {
+                // update time of existing source
+                App.SetSourceTime(sourceId, time);
+                return;
+            }
+
+            // TODO: TEST ME!
+            sourceId = chatView.ID;
+            string iconName = null;
+            string label = null;
+            if (chatView is PersonChatView) {
+                iconName = "smuxi-person-chat";
+                label = chatView.Name;
+            } else if (chatView is GroupChatView) {
+                iconName = "smuxi-group-chat";
+                var nick = GetNick(msg);
+                if (nick == null) {
+                    label = chatView.Name;
+                } else {
+                    label = String.Format("{0} ({1})", chatView.Name, nick);
+                }
+            }
+
+            var theme = Gtk.IconTheme.Default;
+            GLib.Icon icon = null;
+            if (Frontend.HasSystemIconTheme &&
+                iconName != null && theme.HasIcon(iconName)) {
+                icon = new GLib.ThemedIcon(iconName);
+            } else if (iconName != null && theme.HasIcon(iconName)) {
+                // icon wasn't in the system icon theme
+                var iconInfo = theme.LookupIcon(iconName, 256, Gtk.IconLookupFlags.UseBuiltin);
+                if (!String.IsNullOrEmpty(iconInfo.Filename) &&
+                    File.Exists(iconInfo.Filename)) {
+                    icon = new GLib.FileIcon(
+                        GLib.FileFactory.NewForPath(iconInfo.Filename)
+                    );
+                }
+            }
+            App.AppendSource(sourceId, icon, label);
+            App.SetSourceTime(sourceId, time);
+            App.DrawAttention(sourceId);
+            Sources.Add(chatView, sourceId);
+        }
+
+        void DisposeSource(ChatView chatView)
+        {
+            Trace.Call(chatView);
+
+            try {
+                string sourceId;
+                if (!Sources.TryGetValue(chatView, out sourceId)) {
+                    return;
+                }
+
+                App.RemoveSource(sourceId);
+            } finally {
+                Sources.Remove(chatView);
+            }
+        }
+
+        void OnAppActivateSource(object sender, ActivateSourceArgs e)
+        {
+            Trace.Call(sender, e);
+
+            try {
+                MainWindow.PresentWithServerTime();
+
+                ChatView chatView = null;
+                foreach (var kvp in Sources) {
+                    if (kvp.Value != e.SourceId) {
+                        continue;
+                    }
+                    chatView = kvp.Key;
+                }
+                if (chatView == null) {
+                    return;
+                }
+
+                MainWindow.Notebook.CurrentChatView = chatView;
+                DisposeSource(chatView);
+            } catch (Exception ex) {
+#if LOG4NET
+                Logger.Error("OnAppActivateSource(): Exception", ex);
+#endif
+            }
+        }
+#endif
+
         string GetNick(MessageModel msg)
         {
             // HACK: try to obtain the nickname from the message
