@@ -180,10 +180,17 @@ namespace Smuxi.Frontend.Stfl
             bool handled = false;
             if (cmd.IsCommand) {
                 switch (cmd.Command.ToLower()) {
+                    case "help":
+                        CommandHelp(cmd);
+                        break;
                     case "window":
                         CommandWindow(cmd);
                         handled = true;
                         break;
+                    case "exit":
+                        Frontend.Quit();
+                        handled = true;
+                        break;
                     case "gc":
 #if LOG4NET
                         _Logger.Debug("GC.Collect()");
@@ -198,6 +205,29 @@ namespace Smuxi.Frontend.Stfl
             return handled;
         }
 
+        void CommandHelp(CommandModel cmd)
+        {
+            var chatView = f_MainWindow.ChatViewManager.GetChat(cmd.Chat);
+            var builder = new MessageBuilder();
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of commands below
+            builder.AppendHeader(_("Frontend Commands"));
+            chatView.AddMessage(builder.ToMessage());
+
+            string[] help = {
+                "help",
+                "window number",
+                "exit",
+            };
+
+            foreach (string line in help) {
+                builder = new MessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(line);
+                chatView.AddMessage(builder.ToMessage());
+            }
+        }
+
         private void CommandWindow(CommandModel cmd)
         {
             int window;
@@ -304,5 +334,10 @@ namespace Smuxi.Frontend.Stfl
             Text = Text.Substring(0, Position) +
                    Text.Substring(Math.Min(Position + 1, Text.Length));
         }
+
+        static string _(string msg)
+        {
+            return Mono.Unix.Catalog.GetString(msg);
+        }
     }
 }
